# National Hydrologic Geospatial Fabric (hydrofabric) Workflow

## Provisional Citation

Blodgett, D.L., Bock, A.R., Johnson, J.M., Santiago, M., Wieczorek, M.E., 2022, National Hydrologic Geospatial Fabric (hydrofabric) Workflow: U.S. Geological Survey software release, https://doi.org/10.5066/P977IAA2 (pending)

## Overview

This repository houses workflow software for development of three data releases associated with the National Hydrologic Geospatial Fabric (hydrofabric). 

1) Blodgett, D.L., 2022, Updated CONUS river network attributes based on the E2NHDPlusV2 and NWMv2.1 networks: U.S. Geological Survey data release, https://doi.org/10.5066/P9W79I7Q. 

2) Johnson, J.M., Blodgett, D.L., Bock A.R., 2022, Updated CONUS network geometry based on NWMv2.1: U.S. Geological Survey data release, https://doi.org/10.5066/P9ZP6NNO. (pending here https://www.sciencebase.gov/catalog/item/6317a72cd34e36012efa4e8a) 

3) Bock, A.R., Blodgett, D.L., Johnson, J.M., Santiago, M., Wieczorek, M.E., 2022, National Hydrologic Geospatial Fabric Reference and Derived Hydrofabrics: U.S. Geological Survey data release, https://doi.org/10.5066/P9NFPB5S (pending here https://www.sciencebase.gov/catalog/item/60be0e53d34e86b93891012b)

These efforts are development-phase and are working towards the priorities of the USGS  [Water Availability and Use Science Program ](https://www.usgs.gov/programs/water-availability-and-use-science-program) and the NOAA [National Water Model](https://water.noaa.gov/). Details of these efforts will be documented in forthcoming reports and web communication products.

## Repository Layout

The `hyfabric` directory houses an R package that has reused utilities specific to the workflow in the `workspace` directory. The `workspace` directory houses an collection of R-based workflow artifacts that collectively generate the data releases referenced above.

This work is being developed in an open and iterative process. As such, the disclaimer that follows applies. Periodic software releases (tagged and referencable via a DOI) are planned. Please [review the open issues](https://code.usgs.gov/wma/nhgf/reference-hydrofabric/-/issues) for current development priorities.

## Disclaimer

This software is in the public domain because it contains materials that originally came from the U.S. Geological Survey, an agency of the United States Department of Interior. For more information, see the official USGS copyright policy

Although this software program has been used by the U.S. Geological Survey (USGS), no warranty, expressed or implied, is made by the USGS or the U.S. Government as to the accuracy and functioning of the program and related program material nor shall the fact of distribution constitute any such warranty, and no responsibility is assumed by the USGS in connection therewith.

This software is provided "AS IS."

## Artifact Management

In order to keep the workflow system from getting too complex, we will not pursue automated artifact cache management initially. Rather, local builds should (optionally) check for remote content from upstream processing steps and download the output if it is available. *Each processing step will be responsible for fetching its own cache.* Process output should be versioned but can be versioned per processing step such that newly-checked-in code should have the updated version of it's output checked in already and know how to fetch it for other's to use.

Using sciencebase should be considerd a best practice.

This approach will require our dependency graph to remain fairly simple and processing steps to remain fairly large. We should create a process-step diagram and make sure that we are all clear on it's lay out and the strategy for the components we will be implementing.

