library(targets)
library(tarchetypes)

tar_option_set(packages = c("dplyr", "sf", "foreign", "hydroloom", "nhdplusTools"),
               memory = "transient", garbage_collection = TRUE, error = "null", 
               storage = "worker", retrieval = "worker", deployment = "main")

library(future)
library(future.callr)
plan(multisession)

# functions for this workflow
source("R/01_prep_functions.R")
# project utilities
source("R/utils.R")

# user settings
source("R/user_vars.R")

# global configurations
source("R/config.R")

# manual modifications 
source("R/gage_mod.R")

list(
  tar_target(data_paths_file, "cache/data_paths.json", format = "file"),
  tar_target(data_paths, jsonlite::read_json(data_paths_file)),
  ###
  tar_target(usgs_ref_gages_file, data_paths$usgs_nldi_gages, format = "file"),
  tar_target(gages_ii_gages_file, data_paths$gagesii_lyr, format = "file"),
  tar_target(usgs_ref_gages, read_sf(usgs_ref_gages_file)),
  tar_target(gages_ii_ref, filter(usgs_ref_gages,
                                  id %in% filter(read_sf(gages_ii_gages_file), 
                                                 CLASS == "Ref")$STAID)),
  ###
  tar_target(SWIM_dbf_file, data_paths$SWIM_dbf, format = "file"),
  tar_target(SWIM_points_file, data_paths$SWIM_points_path, format = "file"),
  tar_target(SWIM, prepare_SWIM(SWIM_dbf_file, SWIM_points_file, usgs_ref_gages,
                                gages_add, POR_Start, min_da_km_gages, min_obs)),
  ###
  tar_target(NHDPlusV2_gageloc_file, data_paths$gageloc_info, format = "file"),
  tar_target(NHDPlusV2_gdb, data_paths$nhdplus_gdb),
  tar_target(NHDPlusV2_gages, prepare_gage_location(NHDPlusV2_gageloc_file, 
                                                    NHDPlusV2_gdb,
                                                    SWIM,
                                                    usgs_ref_gages,
                                                    min_da_km_gages)),
  ###
  tar_target(combined_waterbodies, make_combined_waterbodies(NHDPlusV2_gdb, proj_crs)),
  tar_target(reference_flowlines, read_sf(data_paths$ref_fl)),
  tar_target(gfv11_gages_rds, data_paths$GFv11_gages_lyr, format = "file"),
  tar_target(gfv11_gages_dbf, file.path(data_paths$data_dir, "GFV11.dbf"), format = "file"),
  tar_target(gfv11_gages, prepare_gfv11_gages(gfv11_gages_rds, gfv11_gages_dbf, usgs_ref_gages,
                                            NHDPlusV2_gages, SWIM, reference_flowlines)),
  ###
  tar_target(final_gages, finalize_gages(SWIM, NHDPlusV2_gages, gfv11_gages, gages_ii_ref, 
                                         gage_info_gpkg, gage_info_csv, gage_info_csvt, 
                                         geo_crs)),
  ###
  tar_render(gage_selection_map_report, "templates/01_Gage_Selection.Rmd", 
             output_file = "../temp/01_Gage_Selection.html",
             params = list(SWIM = SWIM, 
                           NHDPlusV2_gages = NHDPlusV2_gages, 
                           gfv11_gages = gfv11_gages, 
                           final_gages = final_gages,
                           min_da_km_gages = min_da_km_gages)),
  
  tar_target(rpu_vpu, read.csv(system.file("extdata/rpu_vpu.csv", package = "hyfabric"), 
                               colClasses = "character")),
  
  tar_target(vpu_codes, unique(rpu_vpu$vpuid)),
  tar_target(all_rpu_codes, unique(rpu_vpu$rpuid)),
  tar_target(full_cat_file, data_paths$fullcats_table, format = "file"),
  tar_target(full_cat_table, readRDS(full_cat_file)),
  
  tar_target(ref_gpkg, file.path(data_paths$ref_fab_path, 
                                  paste0(vpu_codes, "_reference_features.gpkg")),
             pattern = map(vpu_codes)),
  tar_target(nav_gpkg, file.path("cache", paste0("reference_", vpu_codes,".gpkg")),
             pattern = map(vpu_codes)),
  
  # ~1GB of memory
  tar_target(vpu_base, prepare_vpu_base_layers(ref_gpkg, nav_gpkg, vpu_codes, full_cat_table, rpu_vpu),
             pattern = map(ref_gpkg, nav_gpkg, vpu_codes), deployment = "worker"),
  tar_target(rpu_vpu_out_list, prep_rpu_vpu_out_list(vpu_base, rpu_vpu), pattern = map(vpu_base), 
             deployment = "worker", iteration = "list"),
  tar_target(rpu_vpu_out, prep_rpu_vpu_out(rpu_vpu_out_list, reference_flowlines))
)